# This code is a copy of the Python UserList.py, updated to support
# persistence.

from Persistence import Persistent


class PersistentList (Persistent):

    isAPersistentList = 1
    
    def __init__(self, lst=None):
        if lst is None:
            self.data = []
        else:
            self.data = list(lst)

    def __repr__(self): return repr(self.data)

    def __cmp__(self, other):
        if hasattr(other, 'isAPersistentList'):
            return cmp(self.data, other.data)
        elif type(other) is type([]):
            return cmp(self.data, other)
        else:
            return cmp(self.data, list(other))

    def __len__(self): return len(self.data)

    def __getitem__(self, i): return self.data[i]

    def __setitem__(self, i, item):
        self.data[i] = item
        self._p_changed = 1
        
    def __delitem__(self, i):
        del self.data[i]
        self._p_changed = 1
        
    def __getslice__(self, i, j):
        i = max(i, 0); j = max(j, 0)
        newlist = self.__class__()
        newlist.data = self.data[i:j]
        return newlist

    def __setslice__(self, i, j, other):
        i = max(i, 0); j = max(j, 0)
        if hasattr(other, 'isAPersistentList'):
            self.data[i:j] = other.data
        elif type(other) is type([]):
            self.data[i:j] = other
        else:
            self.data[i:j] = list(other)
        self._p_changed = 1
        
    def __delslice__(self, i, j):
        i = max(i, 0); j = max(j, 0)
        del self.data[i:j]
        self._p_changed = 1

    def __add__(self, other):
        if hasattr(other, 'isAPersistentList'):
            # could we avoid double copy here?
            return self.__class__(self.data + other.data)
        elif type(other) is type([]):
            return self.__class__(self.data + other)
        else:
            return self.__class__(self.data + list(other))

    def __radd__(self, other):
        if hasattr(other, 'isAPersistentList'):
            return self.__class__(other.data + self.data)
        elif type(other) is type([]):
            return self.__class__(other + self.data)
        else:
            return self.__class__(list(other) + self.data)

    def __mul__(self, n):
        return self.__class__(self.data*n)

    __rmul__ = __mul__

    def append(self, item):
        self.data.append(item)
        self._p_changed = 1

    def insert(self, i, item):
        self.data.insert(i, item)
        self._p_changed = 1
        
    def pop(self, i=-1):
        v = self.data.pop(i)
        self._p_changed = 1
        return v
    
    def remove(self, item):
        self.data.remove(item)
        self._p_changed = 1
        
    def count(self, item): return self.data.count(item)

    def index(self, item): return self.data.index(item)

    def reverse(self):
        self.data.reverse()
        self._p_changed = 1
        
    def sort(self, *args):
        apply(self.data.sort, args)
        self._p_changed = 1
        
    def extend(self, other):
        if hasattr(other, 'isAPersistentList'):
            self.data.extend(other.data)
        else:
            self.data.extend(other)
        self._p_changed = 1
